@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: mar  5 2021 (21:40) 
 ## Version: 
-## Last-Updated: Oct  4 2021 (22:30) 
+## Last-Updated: okt 15 2021 (16:09) 
 ##           By: Brice Ozenne
-##     Update #: 200
+##     Update #: 218
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -72,6 +72,9 @@
 ##' residuals(eUN.lmm, format = "wide", plot = "correlation")
 ##' residuals(eUN.lmm, format = "wide", type = "normalized")
 ##' residuals(eUN.lmm, format = "wide", type = "scaled")
+##'
+##' ## residuals and predicted values
+##' residuals(eUN.lmm, data = fitted(eUN.lmm,keep.newdata=TRUE),keep.data=TRUE)
 
 
 ## * residuals.lmm (code)
@@ -125,23 +128,24 @@ residuals.lmm <- function(object, type = "response", format = "long",
                                  "normalized" = "Normalized residuals",
                                  "normalized2" = "Pearson Normalized residuals")
     }
-    ## ** extract
-    if(type.object == "lmm"){
 
+    if(type.object == "lmm"){
+        ## ** update design
         if(!is.null(data)){
             ff.allvars <- c(all.vars(object$formula$mean), all.vars(object$formula$var))
             if(any(ff.allvars %in% names(data) == FALSE)){
                 stop("Incorrect argument \'data\': missing variable(s) \"",paste(ff.allvars[ff.allvars %in% names(data) == FALSE], collapse = "\" \""),"\".\n")
             }
-
+            resdata <- as.data.frame(data)
+            attr(resdata,"original") <- object$data
             design <- .model.matrix.lmm(formula.mean = object$formula$mean.design,
                                         structure = object$design$vcov,
-                                        data = data,
+                                        data = resdata,
                                         var.outcome = object$outcome$var,
                                         var.strata = object$strata$var, U.strata = object$strata$levels,
                                         var.time = object$time$var, U.time = object$time$levels,
                                         var.cluster = object$cluster$var,
-                                        precompute.moments = FALSE
+                                        precompute.moments = !is.null(object$design$precompute.XX)
                                         )
         }else{
             if(keep.data){
@@ -154,10 +158,14 @@ residuals.lmm <- function(object, type = "response", format = "long",
         X <- design$mean
         structure <- design$vcov
         n.cluster <- design$cluster$n
+        precompute.XX <- design$precompute.XX
+        cluster.level <- design$cluster$levels
         index.cluster <- design$index.cluster
-        index.variance <- structure$X$pattern.cluster
         index.time <- design$index.time
-
+        index.variance <- structure$X$pattern.cluster
+        n.pattern <-  NROW(structure$X$Upattern)
+        
+        ## ** update Omega
         if(!is.null(p)){
             if(any(duplicated(names(p)))){
                 stop("Incorrect argument \'p\': contain duplicated names \"",paste(unique(names(p)[duplicated(names(p))]), collapse = "\" \""),"\".\n")
@@ -177,6 +185,7 @@ residuals.lmm <- function(object, type = "response", format = "long",
                 precision <- object$OmegaM1
             }
         }
+        ## ** pre-compute
         sqrtPrecision <- list()
         if("pearson" %in% type.residual){
             sqrtPrecision$pearson <- lapply(Omega,function(iM){1/sqrt(diag(iM))})
@@ -184,16 +193,15 @@ residuals.lmm <- function(object, type = "response", format = "long",
         if("studentized" %in% type.residual || "normalized2" %in% type.residual){
             tX.precision.X <- matrix(0, nrow = NCOL(X), ncol = NCOL(X), dimnames = list(colnames(X),colnames(X)))
 
-            if(!is.null(design$precompute.XX)){
-                n.pattern <- NROW(structure$X$Upattern)
+            if(!is.null(precompute.XX)){
 
                 for (iPattern in 1:n.pattern) { ## iPattern <- 1
                     iOmega <- precision[[iPattern]]
                     iTime <- NCOL(iOmega)
                     iTime2 <- length(iOmega)
 
-                    iX <- matrix(unlist(design$precompute.XX$pattern[[iPattern]]), nrow = iTime2, ncol = dim(design$precompute.XX$pattern[[iPattern]])[3], byrow = FALSE)
-                    tX.precision.X <- tX.precision.X + (as.double(iOmega) %*% iX)[as.double(design$precompute.XX$key)]
+                    iX <- matrix(unlist(precompute.XX$pattern[[iPattern]]), nrow = iTime2, ncol = dim(precompute.XX$pattern[[iPattern]])[3], byrow = FALSE)
+                    tX.precision.X <- tX.precision.X + (as.double(iOmega) %*% iX)[as.double(precompute.XX$key)]
                 }
             }else{
                 for(iId in 1:n.cluster){
@@ -217,19 +225,20 @@ residuals.lmm <- function(object, type = "response", format = "long",
             sqrtPrecision$normalized <- lapply(precision,function(iP){t(chol(iP))})
         }
         
+        ## ** raw residuals
         if(!is.null(data) || !is.null(p)){
             fitted <- X %*% beta
             res <-  as.vector(Y - fitted)
-            M.res <- matrix(NA, nrow = NROW(X), ncol = length(type.residual), dimnames = list(NULL, type.residual))
+            M.res <- matrix(NA, nrow = NROW(X), ncol = length(type.residual), dimnames = list(NULL, name.residual))
         }else{
             fitted <- fitted(object)
             res <- as.vector(object$residuals)
-            M.res <- matrix(NA, nrow = NROW(object$residuals), ncol = length(type.residual), dimnames = list(NULL, type.residual))
+            M.res <- matrix(NA, nrow = NROW(object$residuals), ncol = length(type.residual), dimnames = list(NULL, name.residual))
         }
 
-        ## normalization
+        ## ** normalization
         if ("response" %in% type.residual) {
-            M.res[,"response"] <- res
+            M.res[,"r.response"] <- res
         }
         if (any(type.residual %in% c("studentized", "pearson", "normalized", "normalized2", "scaled"))) {
             for(iId in 1:n.cluster){ ## iId <- 7
@@ -241,27 +250,27 @@ residuals.lmm <- function(object, type = "response", format = "long",
                 for(iType in type.residual){
                     if("pearson" %in% type.residual){
                         resnorm <- iResidual * sqrtPrecision$pearson[[index.variance[iId]]]
-                        M.res[iIndex,"pearson"] <- resnorm[order(iOrder)]
+                        M.res[iIndex,"r.pearson"] <- resnorm[order(iOrder)]
                     }
                     if("studentized" %in% type.residual){
                         iX <- X[iIndex[iOrder],,drop=FALSE]
                         iQ <- iX %*% tX.precision.X.M1 %*% t(iX)
                         resnorm <- iResidual / sqrt(diag(Omega[[index.variance[iId]]] - iQ))
-                        M.res[iIndex,"studentized"] <- resnorm[order(iOrder)]
+                        M.res[iIndex,"r.studentized"] <- resnorm[order(iOrder)]
                     }
                     if("normalized" %in% type.residual){
                         ## resnorm <- as.double(sqrtPrecision$normalized[[index.variance[iId]]] %*% iResidual)
                         resnorm <- as.double(iResidual %*% sqrtPrecision$normalized[[index.variance[iId]]])
-                        M.res[iIndex,"normalized"] <- resnorm[order(iOrder)]
+                        M.res[iIndex,"r.normalized"] <- resnorm[order(iOrder)]
                     }
                     if("normalized2" %in% type.residual){
                         iX <- X[iIndex[iOrder],,drop=FALSE]
                         iQ <- iX %*% tX.precision.X.M1 %*% t(iX)
                         resnorm <- as.double(iResidual %*% t(chol(solve(Omega[[index.variance[iId]]] - iQ))))
-                        M.res[iIndex,"normalized2"] <- resnorm[order(iOrder)]
+                        M.res[iIndex,"r.normalized2"] <- resnorm[order(iOrder)]
                     }
                     if("scaled" %in% type.residual){
-                        M.res[iIndex[iOrder][1],"scaled"] <- iResidual[1]/attr(Omega[[index.variance[iId]]],"sd")[1]
+                        M.res[iIndex[iOrder][1],"r.scaled"] <- iResidual[1]/attr(Omega[[index.variance[iId]]],"sd")[1]
                         if(iN.time>1){
                             for(iTime in 2:iN.time){ ## iTime <- 2
                                 iVar <- Omega[[index.variance[iId]]][iTime,iTime]
@@ -271,7 +280,7 @@ residuals.lmm <- function(object, type = "response", format = "long",
                                 
                                 num <- iResidual[iTime] - iOmega_lk %*% as.double(iPrecision_kk %*% iResidual[1:(iTime-1)])
                                 denom <- iVar - as.double(iOmega_lk %*% iPrecision_kk %*% iOmega_kl)
-                                M.res[iIndex[iOrder][iTime],"scaled"] <- num/sqrt(denom) ## issue in term of dimension
+                                M.res[iIndex[iOrder][iTime],"r.scaled"] <- num/sqrt(denom) ## issue in term of dimension
                             }
                         }
                     }
@@ -279,26 +288,27 @@ residuals.lmm <- function(object, type = "response", format = "long",
             }
         }
 
-        ## add NA
+        ## ** add NA
         if(is.null(match.call()$data) && length(object$index.na)>0){
             inflateNA <-  .addNA(index.na = object$index.na, design = design, time = object$time)
             Msave.res <- M.res
-            M.res <- matrix(NA, nrow = inflateNA$n.allobs, ncol = length(type.residual), dimnames = list(NULL, type.residual))
+            M.res <- matrix(NA, nrow = inflateNA$n.allobs, ncol = length(type.residual), dimnames = list(NULL, name.residual))
             M.res[-object$index.na,] <- Msave.res
 
-            level.cluster <- factor(inflateNA$level.cluster, levels = design$cluster$levels)
+            level.cluster <- factor(inflateNA$level.cluster, levels = cluster.level)
             level.time <- factor(inflateNA$level.time, object$time$levels)
         }else{
-            level.cluster <- factor(design$cluster$levels[index.cluster], levels = design$cluster$levels)
+            level.cluster <- factor(cluster.level[index.cluster], levels = cluster.level)
             level.time <- factor(object$time$levels[index.time], object$time$levels)
         }
 
         ## plot
         ## 
         if(format=="wide"){
+            
             dfL.res <- data.frame(residuals = as.vector(M.res), cluster = level.cluster, time = level.time)
             MW.res <- reshape2::dcast(data = dfL.res,
-                                      formula = cluster~time, value.var = "residuals")
+                                      formula = cluster~time, value.var = "residuals")            
             if(plot=="qqplot"){
                 if(engine.qqplot=="ggplot2"){
                     dfL.res$time <- paste0(object$time$var,": ", dfL.res$time)
@@ -309,9 +319,11 @@ residuals.lmm <- function(object, type = "response", format = "long",
                     m <- NCOL(MW.res)-1
                     sqrtm.round <- ceiling(sqrt(m))
                     sqrtm.round2 <- ceiling(m/sqrtm.round)
+
                     oldpar <- graphics::par(no.readonly = TRUE)   
                     on.exit(graphics::par(oldpar))            
                     graphics::par(mfrow = c(sqrtm.round,sqrtm.round2))
+
                     lapply(1:m,function(iCol){qqtest::qqtest(stats::na.omit(MW.res[,iCol+1]), main = paste0(object$time$var,": ",colnames(MW.res)[iCol+1]," (",label.residual,")"))})
                 }
             }else if(plot == "correlation"){
@@ -338,6 +350,7 @@ residuals.lmm <- function(object, type = "response", format = "long",
             }
 
             if(plot == "none"){
+                names(MW.res)[-1] <- paste0(name.residual,".",names(MW.res)[-1])
                 return(MW.res)
             }else{
                 return(invisible(MW.res))
@@ -361,7 +374,6 @@ residuals.lmm <- function(object, type = "response", format = "long",
             }
 
             if(keep.data){
-                colnames(M.res) <- name.residual
                 out <- cbind(data,M.res)
                 attr(out,"plot") <- attr(M.res,"plot")
             }else if(length(type.residual)==1){
